import { defineStore } from 'pinia';
import { ref, computed } from 'vue';
import { useQuasar } from 'quasar';
import { LocalStorage } from 'quasar';

export interface OfflineAction {
    id: string;
    type: 'add' | 'complete' | 'update' | 'delete';
    itemId?: string;
    data: unknown;
    timestamp: number;
    version?: number;
}

export interface ConflictResolution {
    version: 'local' | 'server' | 'merge';
    action: OfflineAction;
}

export interface ConflictData {
    localVersion: {
        data: Record<string, unknown>;
        timestamp: number;
    };
    serverVersion: {
        data: Record<string, unknown>;
        timestamp: number;
    };
    action: OfflineAction;
}

export const useOfflineStore = defineStore('offline', () => {
    const $q = useQuasar();
    const isOnline = ref(navigator.onLine);
    const pendingActions = ref<OfflineAction[]>([]);
    const isProcessingQueue = ref(false);
    const showConflictDialog = ref(false);
    const currentConflict = ref<ConflictData | null>(null);

    // Initialize from IndexedDB
    const init = () => {
        try {
            const stored = LocalStorage.getItem('offline-actions');
            if (stored) {
                pendingActions.value = JSON.parse(stored as string);
            }
        } catch (error) {
            console.error('Failed to load offline actions:', error);
        }
    };

    // Save to IndexedDB
    const saveToStorage = () => {
        try {
            LocalStorage.set('offline-actions', JSON.stringify(pendingActions.value));
        } catch (error) {
            console.error('Failed to save offline actions:', error);
        }
    };

    // Add a new offline action
    const addAction = (action: Omit<OfflineAction, 'id' | 'timestamp'>) => {
        const newAction: OfflineAction = {
            ...action,
            id: crypto.randomUUID(),
            timestamp: Date.now(),
        };
        pendingActions.value.push(newAction);
        saveToStorage();
    };

    // Process the queue when online
    const processQueue = async () => {
        if (isProcessingQueue.value || !isOnline.value) return;

        isProcessingQueue.value = true;
        const actions = [...pendingActions.value];

        for (const action of actions) {
            try {
                await processAction(action);
                pendingActions.value = pendingActions.value.filter(a => a.id !== action.id);
                saveToStorage();
            } catch (error) {
                if (error instanceof Error && error.message.includes('409')) {
                    $q.notify({
                        type: 'warning',
                        message: 'Item was modified by someone else while you were offline. Please review.',
                        actions: [
                            {
                                label: 'Review',
                                color: 'white',
                                handler: () => {
                                    // TODO: Implement conflict resolution UI
                                }
                            }
                        ]
                    });
                } else {
                    console.error('Failed to process offline action:', error);
                }
            }
        }

        isProcessingQueue.value = false;
    };

    // Process a single action
    const processAction = async (action: OfflineAction) => {
        // TODO: Implement actual API calls
        switch (action.type) {
            case 'add':
                // await api.addItem(action.data);
                break;
            case 'complete':
                // await api.completeItem(action.itemId, action.data);
                break;
            case 'update':
                // await api.updateItem(action.itemId, action.data);
                break;
            case 'delete':
                // await api.deleteItem(action.itemId);
                break;
        }
    };

    // Listen for online/offline status changes
    const setupNetworkListeners = () => {
        window.addEventListener('online', () => {
            (async () => {
                isOnline.value = true;
                await processQueue();
            })().catch(error => {
                console.error('Error processing queue:', error);
            });
        });

        window.addEventListener('offline', () => {
            isOnline.value = false;
        });
    };

    // Computed properties
    const hasPendingActions = computed(() => pendingActions.value.length > 0);
    const pendingActionCount = computed(() => pendingActions.value.length);

    // Initialize
    init();
    setupNetworkListeners();

    const handleConflictResolution = (resolution: ConflictResolution) => {
        // Implement the logic to handle the conflict resolution
        console.log('Conflict resolution:', resolution);
    };

    return {
        isOnline,
        pendingActions,
        hasPendingActions,
        pendingActionCount,
        showConflictDialog,
        currentConflict,
        addAction,
        processQueue,
        handleConflictResolution,
    };
});